@@ -17,16 +17,16 @@
 Castellan exception subclasses
 """
 
-import six.moves.urllib.parse as urlparse
+import urllib
 
-from castellan.openstack.common import _i18n as u
+from castellan.i18n import _
 
 _FATAL_EXCEPTION_FORMAT_ERRORS = False
 
 
 class RedirectException(Exception):
     def __init__(self, url):
-        self.url = urlparse.urlparse(url)
+        self.url = urllib.parse.urlparse(url)
 
 
 class CastellanException(Exception):
@@ -36,16 +36,16 @@ class CastellanException(Exception):
     a 'message' property. That message will get printf'd
     with the keyword arguments provided to the constructor.
     """
-    message = u._("An unknown exception occurred")
+    message = _("An unknown exception occurred")
 
     def __init__(self, message_arg=None, *args, **kwargs):
         if not message_arg:
             message_arg = self.message
         try:
             self.message = message_arg % kwargs
-        except Exception as e:
+        except Exception:
             if _FATAL_EXCEPTION_FORMAT_ERRORS:
-                raise e
+                raise
             else:
                 # at least get the core message out if something happened
                 pass
@@ -53,22 +53,30 @@ class CastellanException(Exception):
 
 
 class Forbidden(CastellanException):
-    message = u._("You are not authorized to complete this action.")
+    message = _("You are not authorized to complete this action.")
 
 
 class KeyManagerError(CastellanException):
-    message = u._("Key manager error: %(reason)s")
+    message = _("Key manager error: %(reason)s")
 
 
 class ManagedObjectNotFoundError(CastellanException):
-    message = u._("Key not found, uuid: %(uuid)s")
+    message = _("Key not found, uuid: %(uuid)s")
+
+
+class InvalidManagedObjectDictError(CastellanException):
+    message = _("Dict has no field '%(field)s'.")
+
+
+class UnknownManagedObjectTypeError(CastellanException):
+    message = _("Type not found, type: %(type)s")
 
 
 class AuthTypeInvalidError(CastellanException):
-    message = u._("Invalid auth_type was specified, auth_type: %(type)s")
+    message = _("Invalid auth_type was specified, auth_type: %(type)s")
 
 
 class InsufficientCredentialDataError(CastellanException):
-    message = u._("Insufficient credential data was provided, either "
-                  "\"token\" must be set in the passed conf, or a context "
-                  "with an \"auth_token\" property must be passed.")
+    message = _("Insufficient credential data was provided, either "
+                "\"token\" must be set in the passed conf, or a context "
+                "with an \"auth_token\" property must be passed.")
