@@ -32,11 +32,6 @@ from castellan.common import utils
 
 _DEFAULT_LOG_LEVELS = ['castellan=WARN']
 
-_DEFAULT_LOGGING_CONTEXT_FORMAT = ('%(asctime)s.%(msecs)03d %(process)d '
-                                   '%(levelname)s %(name)s [%(request_id)s '
-                                   '%(user_identity)s] %(instance)s'
-                                   '%(message)s')
-
 
 def set_defaults(conf, backend=None, barbican_endpoint=None,
                  barbican_api_version=None, auth_endpoint=None,
@@ -46,7 +41,9 @@ def set_defaults(conf, backend=None, barbican_endpoint=None,
                  vault_approle_role_id=None, vault_approle_secret_id=None,
                  vault_kv_mountpoint=None, vault_url=None,
                  vault_ssl_ca_crt_file=None, vault_use_ssl=None,
-                 barbican_endpoint_type=None):
+                 vault_namespace=None,
+                 barbican_endpoint_type=None,
+                 vault_kv_version=None):
     """Set defaults for configuration values.
 
     Overrides the default options values.
@@ -67,8 +64,10 @@ def set_defaults(conf, backend=None, barbican_endpoint=None,
     :param vault_url: Use this for the url for vault.
     :param vault_use_ssl: Use this to force vault driver to use ssl.
     :param vault_ssl_ca_crt_file: Use this for the CA file for vault.
+    :param vault_namespace: Namespace to use for all requests to Vault.
     :param barbican_endpoint_type: Use this to specify the type of URL.
     :                              Valid values are: public, internal or admin.
+    :param vault_kv_version: Use this for the kv version for vault.
     """
     conf.register_opts(key_manager.key_manager_opts, group='key_manager')
 
@@ -134,14 +133,19 @@ def set_defaults(conf, backend=None, barbican_endpoint=None,
         if vault_use_ssl is not None:
             conf.set_default('use_ssl', vault_use_ssl,
                              group=vkm._VAULT_OPT_GROUP)
+        if vault_namespace is not None:
+            conf.set_default('namespace', vault_namespace,
+                             group=vkm._VAULT_OPT_GROUP)
+        if vault_kv_version is not None:
+            conf.set_default('kv_version', vault_kv_version,
+                             group=vkm._VAULT_OPT_GROUP)
 
 
 def enable_logging(conf=None, app_name='castellan'):
     conf = conf or cfg.CONF
 
     log.register_options(conf)
-    log.set_defaults(_DEFAULT_LOGGING_CONTEXT_FORMAT,
-                     _DEFAULT_LOG_LEVELS)
+    log.set_defaults(default_log_levels=_DEFAULT_LOG_LEVELS)
 
     log.setup(conf, app_name)
 
